<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers\Admin;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use App\Validators\AuthValidator;
use App\Managers\User\UserManager;
use App\Http\Controllers\Controller;
use App\Managers\Banner\BannerManager;
use App\Validators\BannerValidator;

class AdminBannersController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->authValidator = new AuthValidator();
        $this->bannerManager = new BannerManager();
        $this->userManager = new UserManager();
        $this->bannerValidator = new BannerValidator();
    }
    
    public function add()
    {
        $users = $this->userManager->pluck('name', 'UUID');
        return view('/admin/banners/add')->with('users', $users);
    }
    
    public function index()
    {
        $banners = $this->bannerManager->select('banner.*','users.name')->leftJoin('users', 'users.UUID', '=', 'banner.user_id')->orderBy('expiry_date', 'DESC')->groupBY('banner.id')->get();
        return view('/admin/banners/list')->with('banners', $banners);
    }
    
    public function delete($UUID)
    {
        $delete = $this->bannerManager->where('UUID', $UUID)->delete();
        if(!$delete)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Banner Deleted Successfully'], 200);
    }
    
    public function save(Request $request)
    {
        if(!$request->all())
        {
            return response()->json(['No input found'], 400);
        }
        
        
        $photo = $request->file('banner');
        $userID = $request->get('user_id');
        $url = $request->get('url');
        $expiry_date = $request->get('expiry_date');
        $validation = $this->bannerValidator->addAdmin($photo, $url, $userID, $expiry_date);
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $expiry_date = date("Y-m-d", strtotime($expiry_date));
        $imageName = rand(1000000,9999999) . date('dmYHis') . '.' . $request->file('banner')->getClientOriginalExtension();
        $request->file('banner')->move(base_path() . "/public/uploads/banner/", $imageName);
        $data['banner_location'] = url('/') . '/uploads/banner/' . $imageName;
        $data['UUID'] = $this->helper->addUuid();
        $data['ip'] = $_SERVER['REMOTE_ADDR'];
        $data['url'] = $request->get('url');
        $data['user_id'] = $userID;
        $data['paypal_id'] = 0;
        $data['url'] = $url;
        $data['expiry_date'] = $expiry_date;
        $res = $this->bannerManager->addAdmin($data);
        if(!$res)
        {
            return response()->json(['Something went wrong'], 400);
        }
        
        return response()->json(['Banner Uploaded SUccefully'], 200);
    }
    
    public function edit(Request $request, $UUID)
    {
        if($request->exists('data'))
        {
            $photo = $request->file('photo');
            
            $data = $request->get('data');
            
            if(!isset($data['position']) || empty($data['position']))
            {
                return redirect()->back()->withErrors(['errors' => 'Position is required'])->withInput();
            }
            $positions = $data['position'];
            
            $validation = $this->playerValidator->update($data, $photo, $UUID);
            if($validation)
            {
                return redirect()->back()->withInput()->withErrors($validation);
            }
            if($photo != null)
            {
                $imageName = rand(1000000,9999999) . date('dmYHis') . '.' . $request->file('photo')->getClientOriginalExtension();
                $request->file('photo')->move(base_path() . "/public/uploads/player/", $imageName);
                $data['photo'] = url('/') . '/uploads/banner/' . $imageName;
            }
           
            $res = $this->playerManager->updateData($data, $UUID);
            if(!$res)
            {
                return redirect()->back()->withInput()->withErrors(array('message' => 'Something went wrong'));
            }
            
            return redirect('/admin/player');
        }
        $banner = $this->bannerManager->where('UUID', $UUID)->first();
        return view('/admin/banners/edit')->with('banner', $banner);
    }
    
    public function update(Request $request, $UUID)
    {
        if(!$request->all())
        {
            return response()->json(['No input found'], 400);
        }
        
        $url = $request->get('url');
        $expiry_date = $request->get('expiry_date');
        $validation = $this->bannerValidator->updateAdmin($url, $expiry_date);
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        
        $data['url'] = $url;
        $expiry_date = date("Y-m-d", strtotime($expiry_date));
        $data['expiry_date'] = $expiry_date;
        $res = $this->bannerManager->where('UUID', $UUID)->update($data);
        if(!$res)
        {
            return response()->json(['Something went wrong'], 400);
        }
        
        return response()->json(['Banner Updated SUccefully'], 200);
    }
}