<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use App\Managers\DecisionPolls\DecisionPollsManager;
use App\Managers\DecisionPollsOption\DecisionPollsOptionManager;
use App\Managers\DecisionPollsOptionVotes\DecisionPollsOptionVotesManager;
use App\Managers\Paypal\PaypalManager;
use App\Managers\Profile\ProfileManager;

class PollsApiController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->decisionPolls = new DecisionPollsManager();
        $this->decisionPollsOption = new DecisionPollsOptionManager();
        $this->decisionPollsOptionVotes = new DecisionPollsOptionVotesManager();
        $this->paypal = new PaypalManager();
        $this->profileManager = new ProfileManager();
    }
    
    public function allDecisionPolls(Request $request)
    {
        $profile = $this->profileManager->first();
        $res['hide_results'] = $hidePollResults = $profile->hide_poll_results;
        $votes = $this->paypal->getCurrentUserTotalVotesCount($request->session()->get('general'));
        $polls = $this->decisionPolls->with('options')->where('default_poll', 1)->orderBy('created_at', 'DESC')->get();
        $mainCount = 0;
        if($polls->count() > 0)
        {
            foreach($polls AS $poll)
            {
                $totalVotes = $this->decisionPollsOptionVotes->getTotalLikesForPoll($poll['UUID']);
                
                $options = $poll['options'];
                unset($poll['options']);
                $res[$mainCount]['polls'] = $poll;
                $likeCounts = $this->decisionPollsOptionVotes->getTotalLikesByUser($request->session()->get('general'), $poll['UUID']);
                $res[$mainCount]['vote'] = $votes['total'] - $likeCounts;
                $totalVotesOfAllUsers = $request->session()->get('total_votes');
                if(!is_numeric($totalVotesOfAllUsers))
                {
                    $totalVotesOfAllUsers = 0;
                }
                
                $totalMaxAllowedVotes = ceil($totalVotesOfAllUsers * 30 / 100);
                $totalMaxAllowedVotes = $totalMaxAllowedVotes - $likeCounts;
                //$res['vote'] = $totalMaxAllowedVotes - $likeCounts;
                if($res[$mainCount]['vote'] > $totalMaxAllowedVotes)
                {
                    $res[$mainCount]['vote'] = $totalMaxAllowedVotes;
                }
                $count = 0;
                foreach($options AS $option)
                {
                    $res[$mainCount]['polls']['options'][$count] = $option;
                    if($hidePollResults == 0)
                    {
                        $res[$mainCount]['polls']['options'][$count]['total_votes'] = $this->decisionPollsOptionVotes->getTotalLikesForPollOption($option['UUID']);
                        if($totalVotes != 0)
                        {
                            $res[$mainCount]['polls']['options'][$count]['percentage'] = (($res[$mainCount]['polls']['options'][$count]['total_votes']) / ($totalVotes)) * 100;
                        }
                        else
                        {
                            $res[$mainCount]['polls']['options'][$count]['percentage'] = 0;
                        }
                    }
                    
                    $count++;
                }
                $res[$mainCount]['expired'] = 0;
                if($poll['expiry_date'] < date("Y-m-d"))
                {
                    $res[$mainCount]['expired'] = 1;
                }
                $mainCount++;
            }
        }
        else
        {
            //$mainCount++;
            $res[$mainCount]['vote'] = 0;
        }

        return response()->json($res,200);
    }
    
    public function votePoll(Request $request, $decisionPollsOptionID)
    {
        if(!$request->exists('votes'))
        {
            return response()->json(['message' => 'No Input found'], 400);
            exit;
        }
        $votes = $request->get('votes');
        if(trim($votes) == '' || !is_numeric($votes))
        {
            return response()->json(['message' => 'Vote must be a number'], 400);
            exit;
        }
        $option = $this->decisionPollsOption->where('UUID', $decisionPollsOptionID)->first();
        if(!$option)
        {
            return response()->json(['message' => 'Something went wrong'], 500);
            exit;
        }
        
        $decisionPollID = $option->decision_polls_id;
        $decisionPoll = $this->decisionPolls->where('UUID', $decisionPollID)->first();
        if(!$decisionPoll)
        {
            return response()->json(['message' => 'Something went wrong'], 500);
            exit;
        }
        
        if($decisionPoll['expiry_date'] < date("Y-m-d H:i:s"))
        {
            return response()->json(['message' => 'Poll Expired'], 403);
            exit;
        }

//        $userOption = $this->decisionPollsOptionVotes->where('decision_polls_id', $option->decision_polls_id)->where('user_id', $request->session()->get('general'))->first();
//        if($userOption)
//        {
//            return response()->json(['message' => 'You had already voted'], 403);
//            exit;
//        }
        
        $likeCounts = $this->decisionPollsOptionVotes->getTotalLikesByUser($request->session()->get('general'), $option->decision_polls_id);
       
        
        
        $totalVotesOfAllUsers = $request->session()->get('total_votes');
        if(!is_numeric($totalVotesOfAllUsers))
        {
            $totalVotesOfAllUsers = 0;
        }

        $totalMaxAllowedVotes = ceil($totalVotesOfAllUsers  * 30 / 100);
        if($votes > $totalMaxAllowedVotes)
        {
            return response()->json(['message' => 'You donot have enough votes to vote'], 403);
            exit;
        }
        
        $totalUserVotes = $this->paypal->getCurrentUserTotalVotesCount($request->session()->get('general'));
        
        if(($totalMaxAllowedVotes - $likeCounts) < $votes)
        {
            return response()->json(['message' => 'You donot have enough votes to vote'], 403);
            exit;
        }
        
        $data['UUID'] = $this->helper->addUuid();
        $data['decision_polls_id'] = $option->decision_polls_id;
        $data['decision_polls_option_id'] = $decisionPollsOptionID;
        $data['votes'] = $votes;
        $data['user_id'] = $request->session()->get('general');
        $data['created_at'] = $data['updated_at'] = date("Y-m-d H:i:s");
        
        $res = $this->decisionPollsOptionVotes->insertData($data, $request->session()->get('general'));
        if(!$res)
        {
            return response()->json(['message' => 'Something went wrong'], 500);
            exit;
        }
        
        //$votes = $this->decisionPollsOptionVotes->getCount()
        return response()->json(['message' => 'You have voted successfully'], 200);
        exit;
    }
}